package com.jiuqi.mde;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.jiuqi.mde.MDEAdditionalIndex;
import com.jiuqi.mde.MDEDataInfo;
import com.jiuqi.mde.MDEDimValueSet;
import com.jiuqi.mde.MDEDimension;
import com.jiuqi.mde.MDEException;
import com.jiuqi.mde.MDEField;
import com.jiuqi.mde.MDEFieldSet;
import com.jiuqi.mde.MDEIndex;
import com.jiuqi.mde.MDERecord;
import com.jiuqi.util.DuplicationException;
import java.util.ArrayList;
import java.util.List;

public class MDERecordSet {
  protected MDEDataInfo dataInfo = null;
  
  protected MDEFieldSet fields = null;
  
  protected boolean hasOrder = false;
  
  protected ArrayList records = new ArrayList(40);
  
  protected MDEIndex dimValuesIndex = null;
  
  protected ArrayList additionalIndexes = new ArrayList();
  
  private boolean isUseLoadDataSync = false;
  
  protected Object mutexLoadData = new Object();
  
  protected Object mutexBuildIndex = new Object();
  
  public MDERecordSet(MDEDataInfo dataInfo) {
    this.dataInfo = dataInfo;
    this.fields = dataInfo.fields();
    this.hasOrder = dataInfo.dimensions().hasOrder();
  }
  
  public void setIsUseLoadDataSync(boolean flag) {
    this.isUseLoadDataSync = flag;
  }
  
  public boolean getIsUseLoadDataSync() {
    return this.isUseLoadDataSync;
  }
  
  public final String getName() {
    return this.dataInfo.getName();
  }
  
  public final MDEDataInfo getDataInfo() {
    return this.dataInfo;
  }
  
  public final int size() {
    return this.records.size();
  }
  
  public final MDERecord get(int index) {
    return this.records.get(index);
  }
  
  public final void add(MDERecord record) throws DuplicationException, MDEException {
    if (this.dimValuesIndex != null && (record.dimValues != null || !record.isInserted())) {
      if (this.hasOrder)
        this.dimValuesIndex.genOrder(record.getDimValues()); 
      addToDVIndex(record);
    } 
    this.records.add(record);
  }
  
  public final void remove(int index) {
    MDERecord record = this.records.get(index);
    if (this.dimValuesIndex != null && record.dimValues != null)
      this.dimValuesIndex.remove(record.getDimValues()); 
    record.emptyRecord();
    this.records.remove(index);
  }
  
  public final void remove(MDERecord record) {
    if (this.dimValuesIndex != null && record.dimValues != null)
      this.dimValuesIndex.remove(record.getDimValues()); 
    record.emptyRecord();
    this.records.remove(record);
    if (this.additionalIndexes != null)
      this.additionalIndexes.clear(); 
  }
  
  public final void removeAll(List<MDERecord> removeRecords) {
    for (int i = 0; i < removeRecords.size(); i++)
      remove(removeRecords.get(i)); 
  }
  
  protected final void addToDVIndex(MDERecord record) throws DuplicationException, MDEException {
    record.refreshDimValues(null);
    if (!this.dataInfo.isUnique()) {
      this.dimValuesIndex.addlast(record.getDimValues(), record);
    } else {
      try {
        this.dimValuesIndex.genOrder(record.getDimValues());
        this.dimValuesIndex.add(record.getDimValues(), record);
      } catch (DuplicationException ex) {
        if (this.hasOrder) {
          record.getDimValues().setValue("顺序号", null);
          this.dimValuesIndex.genOrder(record.getDimValues());
          this.dimValuesIndex.add(record.getDimValues(), record);
          return;
        } 
        throw ex;
      } 
    } 
  }
  
  protected final void buildDimValuesIndex() throws MDEException {
    if (getIsUseLoadDataSync()) {
      synchronized (this.mutexBuildIndex) {
        if (this.dimValuesIndex == null)
          buildDimValuesIndex0(); 
      } 
    } else {
      buildDimValuesIndex0();
    } 
  }
  
  private void buildDimValuesIndex0() throws MDEException {
    if (this.dataInfo.dimensions().size() == 0)
      throw new MDEException("没有定义维度，无法使用键值查找功能"); 
    if (this.dimValuesIndex == null) {
      this.dimValuesIndex = new MDEIndex();
    } else {
      this.dimValuesIndex.clear();
    } 
    try {
      for (int i = 0, count = this.records.size(); i < count; i++) {
        MDERecord record = this.records.get(i);
        addToDVIndex(record);
      } 
    } catch (DuplicationException ex) {
      throw new MDEException(ex);
    } 
  }
  
  protected final MDERecord dvFind(MDEDimValueSet dimValues) throws MDEException {
    if (this.dimValuesIndex == null)
      buildDimValuesIndex(); 
    return (MDERecord)this.dimValuesIndex.findkey(dimValues);
  }
  
  protected final MDERecord dvFindNoOrder(MDEDimValueSet dimValues) throws MDEException {
    if (this.dimValuesIndex == null)
      buildDimValuesIndex(); 
    if (this.hasOrder) {
      int index = this.dimValuesIndex.findNoOrder(dimValues);
      return (MDERecord)this.dimValuesIndex.getValue(index);
    } 
    return (MDERecord)this.dimValuesIndex.findkey(dimValues);
  }
  
  protected final List dvFindMatches(MDEDimValueSet speciafication) throws MDEException {
    if (this.dimValuesIndex == null)
      buildDimValuesIndex(); 
    List result = null;
    if (getIsUseLoadDataSync()) {
      synchronized (this.dimValuesIndex) {
        result = dvFindMatches0(speciafication);
      } 
    } else {
      result = dvFindMatches0(speciafication);
    } 
    return result;
  }
  
  private List dvFindMatches0(MDEDimValueSet speciafication) {
    List<MDERecord> result = null;
    int start = 0, stop = this.dimValuesIndex.size();
    boolean allSpeciafied = false;
    if (speciafication.dimensions().equals(this.dataInfo.dimensions()) && speciafication.getValue(0) != null) {
      int index = -1;
      for (int j = 0, count = speciafication.size(); j < count; j++) {
        if (speciafication.getValue(j) == null) {
          index = j;
          break;
        } 
      } 
      allSpeciafied = (index < 0);
      if (allSpeciafied) {
        start = this.dimValuesIndex.searchfirst(speciafication);
        if (start < 0) {
          start = stop;
        } else {
          stop = this.dimValuesIndex.searchlast(speciafication);
          if (stop < this.dimValuesIndex.size())
            stop++; 
        } 
      } else {
        MDEDimension dimension = speciafication.dimensions().get(index);
        MDEDimValueSet find = (MDEDimValueSet)speciafication.clone();
        find.setValue(index, dimension.getMinValue());
        start = this.dimValuesIndex.searchfirst(find);
        if (start < 0)
          start = -1 - start; 
        find.setValue(index, dimension.getMaxValue());
        stop = this.dimValuesIndex.searchlast(find);
        if (stop < 0) {
          stop = -1 - stop;
        } else {
          stop++;
        } 
        if (stop > this.dimValuesIndex.size())
          stop--; 
      } 
    } 
    result = new ArrayList(stop - start);
    for (int i = start; i < stop; i++) {
      MDERecord record = (MDERecord)this.dimValuesIndex.getValue(i);
      if (allSpeciafied || record.getDimValues().matches(speciafication))
        result.add(record); 
    } 
    return result;
  }
  
  protected final void rebuildFieldIndex(MDEAdditionalIndex index) {
    index.clear();
    index.setCapacity(this.records.size());
    MDEField field = index.getKeyFields().get(0);
    for (int i = 0, count = this.records.size(); i < count; i++) {
      MDERecord record = this.records.get(i);
      index.addlast(record.getFieldValue(field), record);
    } 
  }
  
  protected MDEAdditionalIndex createAdditionalIndex(MDEField field) {
    MDEAdditionalIndex result = new MDEAdditionalIndex();
    result.setField(field);
    return result;
  }
  
  protected final MDEAdditionalIndex getAdditionalIndex(MDEField field) {
    MDEAdditionalIndex result = null;
    for (int i = 0, count = this.additionalIndexes.size(); i < count; i++) {
      MDEAdditionalIndex index = this.additionalIndexes.get(i);
      if (index.keyFieldsMatch(field)) {
        result = index;
        break;
      } 
    } 
    if (result == null) {
      result = createAdditionalIndex(field);
      rebuildFieldIndex(result);
      this.additionalIndexes.add(result);
    } 
    return result;
  }
  
  protected final MDERecord fvFind(MDEField field, Object fieldValue) {
    MDEAdditionalIndex index = getAdditionalIndex(field);
    MDERecord rec = (MDERecord)index.findkey(fieldValue);
    return rec;
  }
  
  public void resetRecords() {
    if (this.additionalIndexes != null)
      this.additionalIndexes.clear(); 
    if (this.dimValuesIndex != null) {
      this.dimValuesIndex.clear();
      this.dimValuesIndex = null;
    } 
    this.records.clear();
  }
}
